% EMDH executes the EMD and Hurst-based speech enhancement algorithm [1].
% This function requires the emd.m file provided by G. Rilling and 
% P. Flandrin at http://perso.ens-lyon.fr/patrick.flandrin/emd.html, and 
% also the LDestimate.m file provided by D. Veitch and P. Abry at
% http://crin.eng.uts.edu.au/~darryl/index.html.
%
% Usage:
%     emdh(infile,outfile,num_IMF,window,H_th)
%            or
%     emdh(infile,outfile) for default parameters
%
% where
%   infile: input .wav filename
%   outfile: output .wav filename
%   num_IMF: total number of modes (IMFs) obtained with EMD (default: 10)
%   window: frame length in samples (default: 512)
%   H_th: Hurst exponent threshold value (default: 0.9)
%
%   Reference:
%      [1] L. Zo, R. Coelho and P. Flandrin, "Speech Enhancement with EMD
%      and Hurst-based Mode Selection", IEEE/ACM Transactions on Audio, 
%      Speech and Language Processing, v. 22, n. 5, pp. 897-909, May 2014.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function emdh(infile,outfile,num_IMF,window,H_th)

if nargin ~= 2 && nargin ~= 5
    fprintf('USAGE: emdh(infile,outfile,num_IMF,window,H_th) \n');
    return;
end   

if nargin == 2
   num_IMF = 10;
   window = 512;
   H_th = 0.9;
end

MaxIter = 2000; % defines the maximum number of sifting iterations

[x,Srate,nbits]=wavread(infile);

tam=length(x);
w = hanning(window);
modes = emd(x,'MAXMODES',num_IMF,'MAXITERATIONS',MaxIter);
y = zeros(1,tam);
shift=window/2;
counter = 1;
for i=1:shift:tam-window
    temp = modes(1:num_IMF,i:i+window-1);
    imf_count = 3;
    H = 0;
    while (H < H_th && imf_count < num_IMF)
        imf_count = imf_count + 1;
        alfa = LDestimate(temp(imf_count,:),1,1,8,0,0,0);
        H = abs(alfa-1)/2;
    end;
    wd = sum(temp(1:imf_count,:));
    y(i:i+window-1) = y(i:i+window-1) + wd.*w';
    counter = counter + 1;
end;

i = i+shift;
temp = modes(1:num_IMF,i:tam);
wd = sum(temp(1:imf_count,:));
w(shift+1:window) = ones(shift,1);
y(i:tam) = y(i:tam) + wd.*w(1:tam-i+1)';

wavwrite(y,Srate,nbits,outfile);

end
